/* Copyright 2003  Alexander V. Diemand

    This file is part of MolTalk.

    MolTalk is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolTalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolTalk; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
 
/* vim: set filetype=objc: */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "CompressedFileStream.oh"
#include "String.oh"


static NSString *pathToGZIP=nil;

@implementation CompressedFileStream


+(void)initialize	//@nodoc
{
	/* find out if we have gzip in our PATH */
	char buf[1024];
#ifdef WIN32
	FILE *p = popen("gzip --version","r");
#else
	FILE *p = popen("which gzip","r");
#endif
	int cread = fread(buf,1,1024,p);
	pclose(p);
#ifndef WIN32
	if (cread<=0 || buf[0]=='w')
	{
		[NSException raise:@"Not Found" format:@"gzip is not in your PATH."];
	}
	buf[cread-1]='\0';
	pathToGZIP = RETAIN([NSString stringWithCString:buf]);
#else
	/* WIN32 */
	//printf("gzip returned: %s\n",buf);
	if (cread<=0 || buf[0]!='g')
	{
		[NSException raise:@"Not Found" format:@"gzip is not in your PATH."];
	}
	pathToGZIP = @"gzip";
#endif
}


-(id)init	//@nodoc
{
	[super init];
	return self;
}

-(void)dealloc	//@nodoc
{
	if (fstream)
	{
		pclose(fstream);
		fstream = NULL;
	}
	[super dealloc];
}


/*
 *   close this stream
 */
-(void)close
{
	if (fstream)
	{
		pclose(fstream);
		fstream = NULL;
	}
}


/*
 *   returns TRUE if stream exists
 */
-(BOOL)ok
{
	return fstream != NULL;
}


/*
 *   create stream in readonly mode
 */
+(id)streamFromFile:(NSString*)path
{
	CompressedFileStream *fs = [CompressedFileStream new];
	fs->fstream = popen([[NSString stringWithFormat:@"%@ -dcq %@",pathToGZIP,path] cString], "r");
	if (!fs->fstream)
	{
		return nil;
	}
	int n = fgetc(fs->fstream);
	if (n == EOF) {
		pclose(fs->fstream);
		return nil;
	}
	ungetc(n,fs->fstream);

	return AUTORELEASE(fs);
}


/*
 *   create stream in writeonly mode
 */
+(id)streamToFile:(NSString*)path
{
	CompressedFileStream *fs = [CompressedFileStream new];
	fs->fstream = popen([[NSString stringWithFormat:@"%@ -c > %@",pathToGZIP,path] cString], "w");
	printf("filestream: %d\n",fileno(fs->fstream));
	return AUTORELEASE(fs);
}


/*
 *   create stream in append mode
 */
+(id)streamAppendToFile:(NSString*)path
{
	/* does not make any sense */
	[NSException raise:@"Unsupported" format:@"cannot append to a compressed file."];
	return nil;
}



@end

