/* Copyright 2003  Alexander V. Diemand

    This file is part of MolTalk.

    MolTalk is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolTalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolTalk; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

/* vim: set filetype=objc: */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "privateStructure.oh"
#include "Chain.oh"
#include "privateChain.oh"
#include "ChainFactory.oh"
#include "Residue.oh"
#include "Atom.oh"
#include "Stream.oh"
#include "String.oh"


@implementation Structure (Private)


-(void)writePDBHeaderTo:(Stream *)strout	//@nodoc
{
	char buffer[82];
	int i;

	// to be done!

	
	/* write MODRES entries for all modified residues */
	Chain *chain;
	Residue *residue;
	NSEnumerator *e_res;
	NSEnumerator *e_chain = [self allChains];
	while ((chain = [e_chain nextObject]))
	{
		e_res = [chain allResidues];
		while ((residue = [e_res nextObject]))
		{
			if ([residue isModified])
			{
				char tbuffer[128];
				memset(buffer,32,80);
				buffer[81]='\0';
				buffer[80]='\n';
				buffer[0]='M';buffer[1]='O';buffer[2]='D';buffer[3]='R';buffer[4]='E';buffer[5]='S';
				buffer[16] = [chain code]; /* chain identifier */
				[pdbcode getCString: tbuffer maxLength: 4];
				/* pdb code */
				buffer[7]=tbuffer[0];buffer[8]=tbuffer[1];buffer[9]=tbuffer[2];buffer[10]=tbuffer[3];
				/* residue name */
				[[residue name] getCString: tbuffer maxLength: 3];
				buffer[12]=tbuffer[0];buffer[13]=tbuffer[1];buffer[14]=tbuffer[2];
				/* residue number */
				snprintf(tbuffer, 10, "% 4u    ", [[residue number] intValue]);
				for (i=0;i<4;i++)
				{
					buffer[18+i]=tbuffer[i];
				}
				buffer[22]=[residue subcode];
				/* residue modname */
				[[residue modname] getCString: tbuffer maxLength: 3];
				buffer[24]=tbuffer[0];buffer[25]=tbuffer[1];buffer[26]=tbuffer[2];
				/* modification description */
				[[residue moddescription] getCString: tbuffer maxLength: 41];
				for (i=0; (i<[[residue moddescription]length] && i<41); i++)
				{
					buffer[29+i]=tbuffer[i];
				}
				[strout writeCString: buffer];
			}
		}
	}
}


-(void)writePDBConectTo:(Stream*)strout	//@nodoc
{

	Chain *chain;
	NSEnumerator *e_chain = [chains objectEnumerator];
	while ((chain = [e_chain nextObject]))
	{
		NSEnumerator *e_res = [chain allResidues];
		Residue *residue;
		//,tatom;
		while ((residue = [e_res nextObject]))
		{
			[self writePDBConectResidue: residue to: strout];
		}
		e_res = [chain allHeterogens];
		while ((residue = [e_res nextObject]))
		{
			[self writePDBConectResidue: residue to: strout];
		}
	}
}


-(void)writePDBConectResidue:(Residue*)residue to:(Stream*)strout	//@nodoc
{
	NSEnumerator *e_atm = [residue allAtoms];
	id atom;
	while ((atom = [e_atm nextObject]))
	{
		unsigned int i=0;
		NSEnumerator *e_atm2 = [atom allBondedAtoms];
		id atom2;
		char buffer[82];
		while ((atom2 = [e_atm2 nextObject]))
		{
			if (i==0)
			{
				memset(buffer,32,82);
				buffer[81]='\0';
				buffer[80]='\n';
				sprintf(buffer,"CONECT% 5u",[[atom number] intValue]);
				buffer[11]=' ';
			}
			snprintf(buffer+(i*5)+11,6,"% 5u",[[atom2 number] intValue]);
			i++;
			buffer[(i*5)+11]=' ';
			if (i>=4)
			{
				i=0;
				[strout writeCString: buffer];
			}
		}
		if (i>0)
		{
			[strout writeCString: buffer];
		}
	}
}


-(void)writePDBChain:(Chain*)chain to:(Stream *)strout fromSerial:(unsigned int*)serial	//@nodoc
{
	//NSLog(@"writing chain %@",chain);
	Residue *residue;
	NSEnumerator *e_res = [chain allResidues];
	Residue *lastres = nil;
	while ((residue = [e_res nextObject]))
	{
		if ([residue isModified])
		{
			[self writePDBHeterogen: residue inChain: chain to: strout fromSerial: serial];
		} else {
			[self writePDBResidue: residue inChain: chain to: strout fromSerial: serial];
		}
		lastres=residue;
	} /* all residues */
	
	/* write out TER */
	if (lastres != nil)
	{ /* only if there is an amino acid in this chain */
		char buffer[82];
		char tbuffer[10];
		memset(buffer,32,80);
		buffer[81]='\0';
		buffer[80]='\n';
		buffer[0]='T';buffer[1]='E';buffer[2]='R';
		buffer[21] = [chain code]; /* chain identifier */
		[[lastres name] getCString: tbuffer maxLength: 3]; /* residue name */
		buffer[17] = tbuffer[0]; buffer[18] = tbuffer[1]; buffer[19] = tbuffer[2];
		snprintf(tbuffer, 10, "% 4u    ", [[lastres number] intValue]); /* residue number */
		buffer[22]=tbuffer[0]; buffer[23]=tbuffer[1]; buffer[24]=tbuffer[2]; buffer[25]=tbuffer[3];
		snprintf(buffer+6,6,"% 5u",*serial); buffer[11]=' '; *serial = *serial + 1;
		[strout writeCString: buffer];
	}
	
	e_res = [chain allHeterogens];
	while ((residue = [e_res nextObject]))
	{
		[self writePDBHeterogen: residue inChain: chain to: strout fromSerial: serial];
	} /* all heterogens */
	
	e_res = [chain allSolvent];
	while ((residue = [e_res nextObject]))
	{
		[self writePDBHeterogen: residue inChain: chain to: strout fromSerial: serial];
	} /* all solvent */
	
}


-(void)writePDBResidue:(Residue*)residue inChain:(Chain*)chain to:(Stream*)strout fromSerial:(unsigned int*)serial	//@nodoc
{
	//NSLog(@"writing residue %@",residue);
	char buffer[82];
	char tbuffer[10];
	memset(buffer,32,80);
	buffer[81]='\0';
	buffer[80]='\n';
	buffer[0]='A';buffer[1]='T';buffer[2]='O';buffer[3]='M';
	buffer[21] = [chain code]; /* chain identifier */
	[[residue name] getCString: tbuffer maxLength: 3]; /* residue name */
	buffer[17] = tbuffer[0]; buffer[18] = tbuffer[1]; buffer[19] = tbuffer[2];
	snprintf(tbuffer, 10, "% 4u    ", [[residue number] intValue]); /* residue number */
	buffer[22]=tbuffer[0]; buffer[23]=tbuffer[1]; buffer[24]=tbuffer[2]; buffer[25]=tbuffer[3];
	buffer[56]='1';buffer[57]='.';buffer[58]='0';buffer[59]='0'; /* occupancy */

	NSEnumerator *e_atms = [residue allAtoms];
	Atom *atom;
	while ((atom = [e_atms nextObject]))
	{
		double temperature,x,y,z;
		temperature = [atom temperature];
		x = [atom x];
		y = [atom y];
		z = [atom z];
		[atom setNumber: *serial];
		snprintf(buffer+6,6,"% 5u",*serial); *serial = *serial + 1;
		buffer[11]=' ';
		[[atom name] getCString: tbuffer maxLength: 4];
		unsigned int len=strlen(tbuffer);
		unsigned int i;
		if (len < 4)
		{
			buffer[12]=' ';
			for (i=0;i<len; i++)
			{
				buffer[13+i]=tbuffer[i]; /* include space before name */
			}
			for (;i<4;i++)
			{
				buffer[13+i]=' ';
			}
		} else {
			for (i=0;i<len; i++)
			{
				buffer[12+i]=tbuffer[i];
			}
		}
		sprintf(buffer+30,"% 8.3f",x);
		sprintf(buffer+38,"% 8.3f",y);
		sprintf(buffer+46,"% 8.3f",z);
		buffer[54]=' ';
		sprintf(buffer+60,"%6.2f",temperature);
		buffer[66]=' ';
		[strout writeCString: buffer];
	}
}


-(void)writePDBHeterogen:(Residue*)residue inChain:(Chain*)chain to:(Stream*)strout fromSerial:(unsigned int*)serial	//@nodoc
{
	char buffer[82];
	char tbuffer[10];
	memset(buffer,32,82);
	buffer[81]='\0';
	buffer[80]='\n';
	buffer[0]='H';buffer[1]='E';buffer[2]='T';buffer[3]='A';buffer[4]='T';buffer[5]='M';
	buffer[21] = [chain code]; /* chain identifier */
	[[residue name] getCString: tbuffer maxLength: 3]; /* residue name */
	buffer[17] = tbuffer[0]; buffer[18] = tbuffer[1]; buffer[19] = tbuffer[2];
	snprintf(tbuffer, 10, "% 4u    ", [[residue number] intValue]); /* residue number */
	buffer[22]=tbuffer[0]; buffer[23]=tbuffer[1]; buffer[24]=tbuffer[2]; buffer[25]=tbuffer[3];
	buffer[56]='1';buffer[57]='.';buffer[58]='0';buffer[59]='0'; /* occupancy */

	//printf("%s\n",buffer);
	NSEnumerator *e_atms = [residue allAtoms];
	Atom *atom;
	while ((atom = [e_atms nextObject]))
	{
		double temperature,x,y,z;
		temperature = [atom temperature];
		x = [atom x];
		y = [atom y];
		z = [atom z];
		[atom setNumber: *serial];
		snprintf(buffer+6,6,"% 5u",*serial); *serial = *serial + 1;
		//snprintf(buffer+6,6,"% 5u",[[atom number] intValue]);
		buffer[11]=' ';
		[[atom name] getCString: tbuffer maxLength: 4];
		unsigned int len=strlen(tbuffer);
		unsigned int i;
		if (len < 4)
		{
			buffer[12]=' ';
			for (i=0;i<len; i++)
			{
				buffer[13+i]=tbuffer[i]; /* include space before name */
			}
			for (;i<4;i++)
			{
				buffer[13+i]=' ';
			}
		} else {
			for (i=0;i<len; i++)
			{
				buffer[12+i]=tbuffer[i];
			}
		}
		sprintf(buffer+30,"% 8.3f",x);
		sprintf(buffer+38,"% 8.3f",y);
		sprintf(buffer+46,"% 8.3f",z);
		buffer[54]=' ';
		sprintf(buffer+60,"%6.2f",temperature);
		buffer[66]=' ';
		[strout writeCString: buffer];
	}
}


-(Chain*)addChain:(NSNumber*)p_chain	//@nodoc
{
	Chain *chain = [self getChain:p_chain];
	if (chain == nil)
	{
		chain = [ChainFactory newChainWithCode:[p_chain charValue]];
		[chains addObject:chain];
		[chain setStructure:self];
	}
	return chain;
}


-(void)expdata:(int)p_expdata	//@nodoc
{
	expdata = p_expdata;
}


-(void)resolution:(float)p_resolution	//@nodoc
{
	resolution = p_resolution;
}


-(void)header:(NSString*)p_header	//@nodoc
{
	RETAIN(p_header);
	if (header)
	{
		RELEASE(header);
	}
	header = p_header;
}


-(void)title:(NSString*)p_title	//@nodoc
{
	RETAIN(p_title);
	if (title)
	{
		RELEASE(title);
	}
	title = p_title;
}


-(void)date:(NSCalendarDate*)p_date	//@nodoc
{
	RETAIN(p_date);
	if (date)
	{
		RELEASE(date);
	}
	date = p_date;
}


-(void)revdate:(NSCalendarDate*)p_date	//@nodoc
{
	RETAIN(p_date);
	if (revdate)
	{
		RELEASE(revdate);
	}
	revdate = p_date;
}


-(void)pdbcode:(NSString*)p_pdbcode	//@nodoc
{
	RETAIN(p_pdbcode);
	if (pdbcode)
	{
		RELEASE(pdbcode);
	}
	pdbcode = p_pdbcode;
}


-(void)hetname:(id)name forKey:(NSString*)key	//@nodoc
{
	if (hetnames==nil)
	{
		hetnames = RETAIN([NSMutableDictionary new]);
	}
	[hetnames setObject:name forKey:key];
}


@end

