/* Copyright 2003  Alexander V. Diemand

    This file is part of MolTalk.

    MolTalk is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MolTalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MolTalk; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
 
/* vim: set filetype=objc: */


#include <stdlib.h>
#include <stdio.h>

#include <Foundation/Foundation.h>

#include <StepTalk/StepTalk.h>

#ifdef ADD_SUPPORT_POSTGRESQL
#include "PostgreSQL.oh"
#endif

#include "libmoltalk/MolTalk.oh"


#undef MEMDEBUG

static const char *histidine_banner = "\
            ..                                                          \n\
           ....                   Copyright 2003 Alexander V. Diemand   \n\
            .:..       This is free software under GNU Public License   \n\
            ..-_                                                        \n\
            00#2:                                                       \n\
            400&]                                            ,          \n\
            j#0Wx,                      ad00L,           _a0000m;       \n\
          _d00#83XXXa,              ad0#RN#000m_      _a0##N#00#g,      \n\
        _d00#O]-=++%33Xna,      aa0##X3]|;-*0000Wa _a0##O3]+Q000#m,     \n\
         +?*]-    --=||]%3X4_a0#RO3]|/-     -*00000##O3+|/-  4{:::.     \n\
           -          --=||%3H3]+/-           -*0#X%]=-`      -:;::.    \n\
                          -----`               _00&%:          :::::.   \n\
                                               d0#X]`           :::::.  \n\
                                               00#3|           :;::--   \n\
                                               ::..          .=::..     \n\
                                              .;:..         ,;:-.. .    \n\
                                              :::..       _dy:...       \n\
                                              ;:;::.     _0000aa        \n\
                                              -:::n#00Wad000#WP'        \n\
       http://bioinformatics.org/moltalk          j#00#0000#W!          \n\
                                                   \"?4#00##P'           \n\
                                                       \"!!              \n\
";

static void usage ()
{
	printf("usage: MolTalk <script file>\n");
	printf("\n");
}

/* print out the banner with the histidine */
static void print_banner ()
{
	printf("M o l T a l k                                            Version %s\n",MOLTALK_VERSION);
	printf("%s\n",histidine_banner);
}


int main (int argc, char **argv)
{
	print_banner ();

	CREATE_AUTORELEASE_POOL(pool);

#ifdef MEMDEBUG
        GSDebugAllocationActive(YES);
#endif

	NSString *fname = nil;
	
	NSProcessInfo *procinfo = [NSProcessInfo processInfo];
	if (![procinfo arguments])
	{
		NSLog(@"Unable to get process information and arguments. Abort.");
		RELEASE(pool);
		return -1;
	}

        STEnvironment *stenvironment = [STEnvironment defaultScriptingEnvironment];
#ifdef ADD_SUPPORT_READLINE
	[stenvironment loadModule:@"ReadlineTranscript"];
#else
	[stenvironment loadModule:@"SimpleTranscript"];
#endif
	STEngine *stengine = [STEngine engineForLanguageWithName:@"Smalltalk"];
	[stenvironment setObject:[procinfo arguments] forName:@"ARGS"];

	/* make classes accessible */
#ifdef ADD_SUPPORT_POSTGRESQL
        [stenvironment setObject:[PostgreSQL class] 		forName: @"PostgreSQL" ];
#endif
        [stenvironment setObject:[StructureFactory class] 	forName: @"StructureFactory" ];
        [stenvironment setObject:[ChainFactory class] 		forName: @"ChainFactory" ];
        [stenvironment setObject:[ResidueFactory class] 	forName: @"ResidueFactory" ];
        [stenvironment setObject:[Structure class] 		forName: @"Structure" ];
        [stenvironment setObject:[Chain class] 			forName: @"Chain" ];
        [stenvironment setObject:[Residue class] 		forName: @"Residue" ];
        [stenvironment setObject:[Atom class] 			forName: @"Atom" ];
        [stenvironment setObject:[Matrix class] 		forName: @"Matrix" ];
        [stenvironment setObject:[Matrix44 class] 		forName: @"Matrix44" ];
        [stenvironment setObject:[Matrix53 class] 		forName: @"Matrix53" ];
        [stenvironment setObject:[Selection class] 		forName: @"Selection" ];
        [stenvironment setObject:[PairwiseStrxAlignment class] 	forName: @"PairwiseStrxAlignment" ];
        [stenvironment setObject:[AlPos class] 			forName: @"AlPos" ];
        [stenvironment setObject:[Vector class] 		forName: @"Vector" ];
        [stenvironment setObject:[FileStream class] 		forName: @"FileStream" ];
        [stenvironment setObject:[Coordinates class] 		forName: @"Coordinates" ];
        [stenvironment setObject:[FileStream class] 		forName: @"FileStream" ];
        [stenvironment setObject:[CompressedFileStream class] 	forName: @"CompressedFileStream" ];
	[stenvironment setObject:stenvironment 			forName: @"Environment" ];

	/* install default factories */
	[StructureFactory setDefaultStructureFactory: [StructureFactory class]];
	[ChainFactory setDefaultChainFactory: [ChainFactory class]];
	[ResidueFactory setDefaultResidueFactory: [ResidueFactory class]];
	
	if (argc >= 2)
	{
		fname = [NSString stringWithCString:(argv[1])];
	}
	
	if (fname ==  nil)
	{
		NSLog(@"A file containing the script must be passed. Abort.");
		usage();
		return(-1);
	}

#ifdef MEMDEBUG
	NSLog(@"allocated objects on starting script\n%s",GSDebugAllocationList(NO));
#endif
	NSString *theScript = [NSString stringWithContentsOfFile:fname];
	id result;
	NS_DURING
	result = [stengine executeCode:theScript inEnvironment:stenvironment];
	NS_HANDLER
	/* handle the exception */
	NSLog(@"%@",localException);
	NS_ENDHANDLER
#ifdef MEMDEBUG
	NSLog(@"change of allocated objects\n%s",GSDebugAllocationList(YES));
	NSLog(@"allocated objects on exiting script\n%s",GSDebugAllocationList(NO));
#endif

	fflush(stdout);
	
	RELEASE(pool);
#ifdef MEMDEBUG
	NSLog(@"change of allocated objects\n%s",GSDebugAllocationList(YES));
	NSLog(@"allocated objects after pool release\n%s",GSDebugAllocationList(NO));
#endif

	return 0;
}


